<?php

require_once dirname(__FILE__) . '/BaseTruevoAdminController.php';

class AdminProcessOrderController extends BaseTruevoAdminController
{
    /**
     * Process a pre-authorised order currently on hold.
     */
    public function postProcess()
    {
        $response = $this->truevoApi->capturePayment();

        // Responses
        if ($response) {
            if ($response->outcome->status === "RECEIVED") {
                $transactionUniqueIdentifier = $response->transactionUniqueIdentifier;

                $log = $this->buildLogArray('CAPTURE', $response, ['transaction_unique_identifier' => $transactionUniqueIdentifier]);
                $this->logToOrder($this->order, $log);

                switch ( $this->pollingTransactionStatus($transactionUniqueIdentifier) ) {
                    case 'COMPLETE':
                        $this->order->addOrderPayment(
                            $response->amount->totalAmount,
                            $this->module->displayName . " (Capture)",
                            $transactionUniqueIdentifier,
                            new Currency($this->order->id_currency)
                        );

                        $this->order->setCurrentState(Configuration::get('PS_OS_PREPARATION'));
                        $this->assignStatusNotification('success', 'Truevo payment successfully captured.');
                        break;
                    case 'DECLINED':
                        $this->order->setCurrentState(Configuration::get('PS_OS_ERROR'));
                        $this->assignStatusNotification('danger', 'Truevo payment has been declined.');
                        break;
                    default: 
                        // Payment is still pending
                        $this->assignStatusNotification('info', 'Truevo payment still pending.');
                        break;
                }

                $this->redirectToOrderView($this->order->id);
            }
        }

        // Error capturing payment
        $this->assignStatusNotification('danger', 'There was an error capturing the Truevo payment.');
        $this->redirectToOrderView($this->order->id);
    }
}