<?php

require_once dirname(__FILE__) . '/BaseTruevoAdminController.php';

class AdminReverseOrderController extends BaseTruevoAdminController
{
    /**
     * Reverse a payment.
     */
    public function postProcess()
    {
        $response = $this->truevoApi->reversal();

        // Responses
        if ($response) {
            if ($response->outcome->status === "RECEIVED") {
                $transactionUniqueIdentifier = $response->transactionUniqueIdentifier;

                $log = $this->buildLogArray('REVERSAL', $response, ['transaction_unique_identifier' => $transactionUniqueIdentifier]);
                $this->logToOrder($this->order, $log);

                switch ( $this->pollingTransactionStatus($transactionUniqueIdentifier) ) {
                    case 'COMPLETE':
                        $this->order->setCurrentState(Configuration::get('PS_OS_CANCELED'));
                        $this->assignStatusNotification('success', 'Truevo payment successfully reversed.');
                        break;
                    case 'DECLINED':
                        $this->assignStatusNotification('danger', 'Truevo payment declined the reversal.');
                        break;
                    default: 
                        // Reversal is still pending
                        $this->assignStatusNotification('info', 'Reversal is still pending.');
                        break;
                }

                $this->redirectToOrderView($this->order->id);
            }
        }

        // Error capturing payment
        $this->assignStatusNotification('danger', 'There was an error requesting a reversal for the Truevo payment.');
        $this->redirectToOrderView($this->order->id);
    }
}