<?php

class TruevoPaymentsCallbackModuleFrontController extends ModuleFrontController
{
    public function initContent()
    {
        parent::initContent();
        $this->postProcess();
        $this->setTemplate('module:truevopayments/views/templates/front/payment_description.tpl');
    }

    public function postProcess()
	{
        if ($_SERVER['REQUEST_METHOD'] === 'POST') {
            $cart = $this->context->cart;
            $authorized = false;

            /**
             * Verify if this module is enabled and if the cart has
             * a valid customer, delivery address and invoice address
             */
            if (!$this->module->active || $cart->id_customer == 0 || $cart->id_address_delivery == 0 || $cart->id_address_invoice == 0) {
                Tools::redirect($this->module->getOrderUrlPath());
            }

            /**
             * Verify if this payment module is authorized
             */
            foreach (Module::getPaymentModules() as $module) {
                if ($module['name'] == 'truevopayments') {
                    $authorized = true;
                    break;
                }
            }

            if (!$authorized) {
                die($this->l('This payment method is not available.'));
            }

            /** @var CustomerCore $customer */
            $customer = new Customer($cart->id_customer);

            /**
             * Check if this is a valid customer account
             */
            if (!Validate::isLoadedObject($customer)) {
                Tools::redirect($this->module->getOrderUrlPath());
            }

            if ($_POST['transactionStatus'] === 'COMPLETE') {
                if ($_POST['transactionType'] === "AUTHORIZE" || $this->module->isPreauthEnabled()) {
                    $this->setOrder(Configuration::get('TRUEVO_ON_HOLD'));
                } else {
                    $this->setOrder(Configuration::get('PS_OS_PAYMENT'));
                }

                Tools::redirect($this->context->link->getPageLink('order-confirmation', true, null, array(
                    'id_cart'   => $cart->id,
                    'id_module' => $this->module->id,
                    'id_order'  => $this->module->currentOrder,
                    'key'       => $customer->secure_key
                )));
            } elseif ($_POST['transactionStatus'] === 'PENDING') {
                $this->setOrder(Configuration::get('PS_OS_OUTOFSTOCK_UNPAID'));

                Tools::redirect($this->context->link->getPageLink('order-confirmation', true, null, array(
                    'id_cart'   => $cart->id,
                    'id_module' => $this->module->id,
                    'id_order'  => $this->module->currentOrder,
                    'key'       => $customer->secure_key
                )));
            } else {
                $this->setOrder(Configuration::get('PS_OS_ERROR'));

                if ($_POST["reasonDescription"] == null) {
                    $reason = $_POST["transactionStatus"] === "DECLINED" ? "Your payment has been declined." : "An unexpected error occured.";

                    if (!$customer->is_guest) {
                        $reason .= " You can attempt to reorder by visiting your order history.";
                    }
                } else {
                    $reason = $_POST["reasonDescription"];
                }

                $this->errors[] = $reason;

                $this->redirectWithNotifications($this->module->getOrderUrlPath(3));
            }
        }
    }

    private function setOrder($orderState)
    {
        $this->setTransactionDetails();

        $this->module->validateOrder(
            (int) $this->context->cart->id,
            $orderState,
            (float) $this->context->cart->getOrderTotal(true, Cart::BOTH),
            $this->module->displayName,
            null,
            array('transaction_id' => $_POST["transactionId"]),
            (int) $this->context->currency->id,
            false,
            $this->context->customer->secure_key
        );
    }

    private function setTransactionDetails()
    {
        $shop = new Shop($this->context->cart->id_shop);

        $transactionType = !is_null($_POST['transactionType']) ? $_POST['transactionType'] : "Unknown";
        $rrn = !is_null($_POST['rrn']) ? $_POST['rrn'] : "Unknown";
        $transactionId = !is_null($_POST['transactionId']) ? $_POST['transactionId'] : "Unknown";
        $transactionStatus = !is_null($_POST['transactionStatus']) ? $_POST['transactionStatus'] : "Unknown";
        $responseCode = !is_null($_POST['responseCode']) ? $_POST['responseCode'] : "Unknown";
        $merchantReference = !is_null($_POST['merchantReference']) ? $_POST['merchantReference'] : "Unknown";

        $data = [
            'id_cart'            => $this->context->cart->id,
            'id_order'           => null,
            'transaction_type'   => strtoupper(bqSQL($transactionType)),
            'rrn'                => bqSQL($rrn),
            'transaction_id'     => bqSQL($transactionId),
            'transaction_status' => bqSQL($transactionStatus),
            'response_code'      => bqSQL($responseCode),
            'merchant_reference' => bqSQL($merchantReference),
            'payment_reference'  => $shop->name . ' Purchase',
        ];

        $result = Db::getInstance()->insert('truevo_payment_response_details', $data, true);

        if (!$result) {
            $this->module->truevoLogger('Truevo payment response failed to save transaction details. Check log file for more details.');
            $this->module->truevoFileLogger('Truevo payment response transaction details failed to save. ' . print_r($data, true));
        }
    }
}